<ul>
    {{#children}}
        <li>

            {{#has_children}}
                <input id="{{uid}}" type="checkbox" {{#traversed}}checked{{/traversed}} />
            {{/has_children}}

            {{#request_path}}
                <a href="{{request_path}}">{{name}}</a>
            {{/request_path}}
            {{^request_path}}
                <label for="{{uid}}">{{name}}</label>
            {{/request_path}}

            {{#has_children}}
                {{>menu}}
            {{/has_children}}

        </li>
    {{/children}}
</ul>