<!DOCTYPE html>

<html>
    <head>
        <meta charset="utf-8"/>
        <meta name="viewport" content="initial-scale=1" />

        <title>{{title}}</title>

        <link rel="stylesheet" type="text/css" href="/main.css" />
        <link rel="stylesheet" type="text/css" href="/pygmentize-codehilite.css" />

        {{#config.css}}
            <link rel="stylesheet" type="text/css" href="{{.}}" />
        {{/config.css}}

        <script src="/relative-time.js"></script>
    </head>

    <body>
        {{>header-and-menu}}

        <div class="main">
            {{#file.version}}
                <div class="version">
                    Version <abbr title="{{hash}}">{{short_hash}}</abbr>
                    committed <time datetime="{{date}}">{{date}}</time>
                    by {{author.name}}.
                </div>
            {{/file.version}}

            <div class="content">
                {{{content}}}
            </div>
        </div>
    </body>
</html>