<!DOCTYPE html>

<html>
    <head>
        <meta charset="utf-8"/>

        <title>{{title}}</title>

        <link rel="stylesheet" type="text/css" href="/main.css" />

        <script src="/relative-time.js"></script>
    </head>

    <body>
        {{>header-and-menu}}

        <div class="main">
            <div class="content">
                <h1>Search results</h1>

                {{#search_results}}
                    <div>
                        <a href="{{request_path}}">{{name}}</a>
                        {{#version}}
                            <div class="version" style="text-align: left;">
                                Version <abbr title="{{hash}}">{{short_hash}}</abbr>
                                committed <time datetime="{{date}}">{{date}}</time>
                                by {{author.name}}.
                            </div>
                        {{/version}}
                    </div>
                    <hr />
                {{/search_results}}

                {{^search_results}}
                    <p>
                        None.
                    </p>
                {{/search_results}}
            </div>
        </div>
    </body>
</html>