{{#root}}
    <div class="header">
        {{#index}}
            <a class="title" href="{{request_path}}">{{name}}</a>
        {{/index}}
        <form method="post" class="search">
            <input type="hidden" name="action" value="search" />
            <input type="text" name="terms" placeholder="search" value="{{search_terms}}" />
        </form>
        {{#auth}}
            {{#authenticated}}
                <span class="auth">
                    {{#display_name}}
                        <span class="auth_display_name">{{display_name}}</span> ::
                    {{/display_name}}
                    <a href="?logout=yes">Log out</a>
                </span>
            {{/authenticated}}
            {{^authenticated}}
                {{#available}}
                    <a href="?login=yes">Log in</a>
                {{/available}}
            {{/authenticated}}
        {{/auth}}
    </div>

    <div class="menu">
        {{>menu}}
    </div>
{{/root}}