body {

    min-width: 1440px;

    margin: 0;
    padding: 48px 0 0 0;

    font-family: sans-serif;
    color: #222;
    background-color: #ececec;

    display: grid;
    grid-template-columns: max-content 1fr;
    gap: 16px;
}

/* https://code.iamkate.com/html-and-css/fixing-browsers-broken-monospace-font-handling/ */
pre, code, kbd, samp, tt {
  font-family: monospace, monospace;
  font-size: 1em;
}

.header {
    z-index: 1;
    position: fixed;
    top: 0;
    left: 0;
    right: 0;
    height: 48px;
    padding: 0 32px;
    color: #f8f8f8;
    background-color: #282838;
    filter: drop-shadow(0px 0px 6px #666);
    display: flex;
    justify-content: space-between;
    font-size: 24px;
    line-height: 48px;
}

.header a {
    color: #f8f8f8;
    text-decoration: none;
}

.header .title {
    flex-shrink: 0;
}

.header .search {
    flex-grow: 1;
}

.header .search > input {
    appearance: none;
    border: none;
    background-color: transparent;

    width: calc(100% - 64px);
    height: 48px;
    margin: 0 32px;

    font-family: sans-serif;
    font-size: 24px;
    color: #f8f8f8;
}

.header .search > input::placeholder {
    color: gray;
    font-style: italic;
}

.menu {
    grid-column: 1;
    padding: 0 24px;
}

.main {
    grid-column: 2;
    margin: 32px;
}

.version {
    text-align: center;
    color: #aaa;
}

.content > * {
    margin: 24px;
}

h1, h2, h3, h4, h5, h6, th {
    text-align: left;
    color: #383f38;
    background-color: #f8f8f8;
}

h1, h2, h3, h4, h5, h6{
    margin: 32px 12px 12px 12px !important;
    padding: 6px 12px;
    border: 1px solid #ccc;
    filter: drop-shadow(0px 0px 6px #ccc);
}

hr {
    color: #efefef;
}

a {
    color: blue;
}

p {
    line-height: 1.4rem;
}

table, pre, details {
    background-color: #f2f2f2;
    border: 1px solid #ccc;
    filter: drop-shadow(0px 0px 6px #ccc);
}

table {
    border-collapse: collapse;
}

th, td {
    min-width: 64px;
    padding: 5px 10px;
    border: 1px solid #ccc;
}

pre {
    padding: 6px 12px;
}

code {
    padding: 0 3px;
    margin: 0 3px;
    background-color: #f8dec6;
    border: 1px solid #f6b472;
    border-radius: 4px;
    white-space: pre;
}

pre code {
    padding: 0;
    margin: 0;
    background: none;
    border: none;
}

details {
    border-radius: 10px;
    padding: 10px 10px 0 10px;
}

summary {
    cursor: pointer;
    margin-bottom: 10px;
}

li {
    padding-top: 3px;
}

img, video {
    max-width: 100%;
}

.menu ul {
    list-style: none; /* Remove default bullets */
    padding-left: 12px;
}

.menu li > input {
    appearance: none;
    -webkit-appearance: none;
    outline: none;
    width: 0;
    height: 0;
    border-top: 8px solid transparent;
    border-bottom: 8px solid transparent;
    border-left: 8px solid #888;
    cursor: pointer;
    vertical-align: middle;
    transition: transform .2s;
}

.menu li > input:focus, .menu li > input:active {
    border-left: 8px solid #000;
}

.menu li > input:checked {
    transform: rotate(45deg);
}

.menu li > input:not(:checked) ~ ul {
    display: none;
}

.menu label {
    cursor: pointer;
    text-decoration: underline;
}

.menu a:only-child {
    margin-left: 20px;
}

.codehilitetable td {
    padding: 0;
}